"""Models for the extracted item details"""

from moviebox_api.extractor.models.json import ItemJsonDetailsModel
from moviebox_api.extractor.models.tag import ItemTagDetailsModel

__all__ = ["ItemJsonDetailsModel", "ItemTagDetailsModel"]
